@@ -1,6 +1,6 @@
-// 
+//
 // Sysinfo
-// 
+//
 // Copyright (c) 2015 Guillaume Gomez
 //
 
@@ -11,14 +11,17 @@ use sys::processor::*;
 use sys::process::*;
 use sys::disk::{self, Disk, DiskType};
 
-use ::{ComponentExt, DiskExt, ProcessExt, ProcessorExt, SystemExt};
+use ::{ComponentExt, DiskExt, ProcessExt, ProcessorExt, RefreshKind, SystemExt};
 
+use std::borrow::Borrow;
 use std::cell::UnsafeCell;
 use std::collections::HashMap;
 use std::ffi::{OsStr, OsString};
+use std::mem::MaybeUninit;
+use std::ops::Deref;
 use std::os::unix::ffi::OsStringExt;
 use std::sync::Arc;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 use sys::processor;
 use std::{fs, mem, ptr};
 
@@ -27,6 +30,8 @@ use libc::{self, c_void, c_int, size_t, c_char, sysconf, _SC_PAGESIZE};
 use utils;
 use Pid;
 
+use std::process::Command;
+
 use rayon::prelude::*;
 
 /// Structs containing system's information.
@@ -43,6 +48,7 @@ pub struct System {
     disks: Vec<Disk>,
     network: NetworkData,
     uptime: u64,
+    port: ffi::mach_port_t,
 }
 
 impl Drop for System {
@@ -108,17 +114,17 @@ unsafe fn ultostr(s: *mut c_char, val: u32) {
 
 unsafe fn perform_call(conn: ffi::io_connect_t, index: c_int, input_structure: *mut ffi::KeyData_t,
                        output_structure: *mut ffi::KeyData_t) -> i32 {
-    let mut structure_output_size = ::std::mem::size_of::<ffi::KeyData_t>();
+    let mut structure_output_size = mem::size_of::<ffi::KeyData_t>();
 
     ffi::IOConnectCallStructMethod(conn, index as u32,
-                                   input_structure, ::std::mem::size_of::<ffi::KeyData_t>(),
+                                   input_structure, mem::size_of::<ffi::KeyData_t>(),
                                    output_structure, &mut structure_output_size)
 }
 
 unsafe fn read_key(con: ffi::io_connect_t, key: *mut c_char) -> Result<ffi::Val_t, i32> {
-    let mut input_structure: ffi::KeyData_t = ::std::mem::zeroed::<ffi::KeyData_t>();
-    let mut output_structure: ffi::KeyData_t = ::std::mem::zeroed::<ffi::KeyData_t>();
-    let mut val: ffi::Val_t = ::std::mem::zeroed::<ffi::Val_t>();
+    let mut input_structure: ffi::KeyData_t = mem::zeroed::<ffi::KeyData_t>();
+    let mut output_structure: ffi::KeyData_t = mem::zeroed::<ffi::KeyData_t>();
+    let mut val: ffi::Val_t = mem::zeroed::<ffi::Val_t>();
 
     input_structure.key = strtoul(key, 4, 16);
     input_structure.data8 = ffi::SMC_CMD_READ_KEYINFO;
@@ -141,7 +147,7 @@ unsafe fn read_key(con: ffi::io_connect_t, key: *mut c_char) -> Result<ffi::Val_
     } else {
         libc::memcpy(val.bytes.as_mut_ptr() as *mut c_void,
                      output_structure.bytes.as_mut_ptr() as *mut c_void,
-                     ::std::mem::size_of::<[u8; 32]>());
+                     mem::size_of::<[u8; 32]>());
         Ok(val)
     }
 }
@@ -162,7 +168,7 @@ unsafe fn get_unchecked_str(cp: *mut u8, start: *mut u8) -> String {
     let len = cp as usize - start as usize;
     let part = Vec::from_raw_parts(start, len, len);
     let tmp = String::from_utf8_unchecked(part.clone());
-    ::std::mem::forget(part);
+    mem::forget(part);
     tmp
 }
 
@@ -217,9 +223,10 @@ fn get_disk_types() -> HashMap<OsString, DiskType> {
             if next_media == 0 {
                 break;
             }
-            let mut props = mem::uninitialized();
-            let result = ffi::IORegistryEntryCreateCFProperties(next_media, &mut props,
+            let mut props = MaybeUninit::<ffi::CFMutableDictionaryRef>::uninit();
+            let result = ffi::IORegistryEntryCreateCFProperties(next_media, props.as_mut_ptr(),
                                                                 ffi::kCFAllocatorDefault, 0);
+            let props = props.assume_init();
             if result == ffi::KERN_SUCCESS as i32 && check_value(props, b"Whole\0") {
                 let mut name: ffi::io_name_t = mem::zeroed();
                 if ffi::IORegistryEntryGetName(next_media,
@@ -232,7 +239,7 @@ fn get_disk_types() -> HashMap<OsString, DiskType> {
                                    DiskType::SSD
                                });
                 }
-                ffi::CFRelease(props as *mut c_void);
+                ffi::CFRelease(props as *mut _);
             }
             ffi::IOObjectRelease(next_media);
         }
@@ -266,8 +273,8 @@ fn get_disks() -> Vec<Disk> {
 }
 
 fn get_uptime() -> u64 {
-    let mut boottime: libc::timeval = unsafe { ::std::mem::zeroed() };
-    let mut len = ::std::mem::size_of::<libc::timeval>();
+    let mut boottime: libc::timeval = unsafe { mem::zeroed() };
+    let mut len = mem::size_of::<libc::timeval>();
     let mut mib: [c_int; 2] = [libc::CTL_KERN, libc::KERN_BOOTTIME];
     unsafe {
         if libc::sysctl(mib.as_mut_ptr(),
@@ -285,6 +292,30 @@ fn get_uptime() -> u64 {
     unsafe { libc::difftime(csec, bsec) as u64 }
 }
 
+fn parse_command_line<T: Deref<Target = str> + Borrow<str>>(cmd: &[T]) -> Vec<String> {
+    let mut x = 0;
+    let mut command = Vec::with_capacity(cmd.len());
+    while x < cmd.len() {
+        let mut y = x;
+        if cmd[y].starts_with('\'') || cmd[y].starts_with('"') {
+            let c = if cmd[y].starts_with('\'') {
+                '\''
+            } else {
+                '"'
+            };
+            while y < cmd.len() && !cmd[y].ends_with(c) {
+                y += 1;
+            }
+            command.push(cmd[x..y].join(" "));
+            x = y;
+        } else {
+            command.push(cmd[x].to_owned());
+        }
+        x += 1;
+    }
+    command
+}
+
 struct Wrap<'a>(UnsafeCell<&'a mut HashMap<Pid, Process>>);
 
 unsafe impl<'a> Send for Wrap<'a> {}
@@ -295,7 +326,7 @@ fn update_process(wrap: &Wrap, pid: Pid,
                   mib: &mut [c_int], mut size: size_t) -> Result<Option<Process>, ()> {
     let mut proc_args = Vec::with_capacity(size as usize);
     unsafe {
-        let mut thread_info = ::std::mem::zeroed::<libc::proc_threadinfo>();
+        let mut thread_info = mem::zeroed::<libc::proc_threadinfo>();
         let (user_time, system_time, thread_status) = if ffi::proc_pidinfo(pid,
                              libc::PROC_PIDTHREADINFO,
                              0,
@@ -308,8 +339,12 @@ fn update_process(wrap: &Wrap, pid: Pid,
             (0, 0, None)
         };
         if let Some(ref mut p) = (*wrap.0.get()).get_mut(&pid) {
+            if p.memory == 0 { // We don't have access to this process' information.
+                force_update(p);
+                return Ok(None);
+            }
             p.status = thread_status;
-            let mut task_info = ::std::mem::zeroed::<libc::proc_taskinfo>();
+            let mut task_info = mem::zeroed::<libc::proc_taskinfo>();
             if ffi::proc_pidinfo(pid,
                                  libc::PROC_PIDTASKINFO,
                                  0,
@@ -326,13 +361,45 @@ fn update_process(wrap: &Wrap, pid: Pid,
             return Ok(None);
         }
 
-        let mut task_info = ::std::mem::zeroed::<libc::proc_taskallinfo>();
+        let mut task_info = mem::zeroed::<libc::proc_taskallinfo>();
         if ffi::proc_pidinfo(pid,
                              libc::PROC_PIDTASKALLINFO,
                              0,
                              &mut task_info as *mut libc::proc_taskallinfo as *mut c_void,
                              taskallinfo_size as i32) != taskallinfo_size as i32 {
-            return Err(());
+            match Command::new("/bin/ps") // not very nice, might be worth running a which first.
+                          .arg("wwwe")
+                          .arg("-o")
+                          .arg("ppid=,command=")
+                          .arg(pid.to_string().as_str())
+                          .output() {
+                Ok(o) => {
+                    let o = String::from_utf8(o.stdout).unwrap_or_else(|_| String::new());
+                    let o = o.split(' ').filter(|c| !c.is_empty()).collect::<Vec<_>>();
+                    if o.len() < 2 {
+                        return Err(());
+                    }
+                    let mut command = parse_command_line(&o[1..]);
+                    if let Some(ref mut x) = command.last_mut() {
+                        **x = x.replace("\n", "");
+                    }
+                    let p = match i32::from_str_radix(&o[0].replace("\n", ""), 10) {
+                        Ok(x) => x,
+                        _ => return Err(()),
+                    };
+                    let mut p = Process::new(pid, if p == 0 { None } else { Some(p) }, 0);
+                    p.exe = PathBuf::from(&command[0]);
+                    p.name = match p.exe.file_name() {
+                        Some(x) => x.to_str().unwrap_or_else(|| "").to_owned(),
+                        None => String::new(),
+                    };
+                    p.cmd = command;
+                    return Ok(Some(p));
+                }
+                _ => {
+                    return Err(());
+                }
+            }
         }
 
         let parent = match task_info.pbsd.pbi_ppid as Pid {
@@ -387,11 +454,11 @@ fn update_process(wrap: &Wrap, pid: Pid,
                         &mut size, ::std::ptr::null_mut(), 0) != -1 {
             let mut n_args: c_int = 0;
             libc::memcpy((&mut n_args) as *mut c_int as *mut c_void, ptr as *const c_void,
-                         ::std::mem::size_of::<c_int>());
-            let mut cp = ptr.offset(::std::mem::size_of::<c_int>() as isize);
+                         mem::size_of::<c_int>());
+            let mut cp = ptr.add(mem::size_of::<c_int>());
             let mut start = cp;
-            if cp < ptr.offset(size as isize) {
-                while cp < ptr.offset(size as isize) && *cp != 0 {
+            if cp < ptr.add(size) {
+                while cp < ptr.add(size) && *cp != 0 {
                     cp = cp.offset(1);
                 }
                 p.exe = Path::new(get_unchecked_str(cp, start).as_str()).to_path_buf();
@@ -407,13 +474,13 @@ fn update_process(wrap: &Wrap, pid: Pid,
                         need_root = false;
                     }
                 }
-                while cp < ptr.offset(size as isize) && *cp == 0 {
+                while cp < ptr.add(size) && *cp == 0 {
                     cp = cp.offset(1);
                 }
                 start = cp;
                 let mut c = 0;
                 let mut cmd = Vec::new();
-                while c < n_args && cp < ptr.offset(size as isize) {
+                while c < n_args && cp < ptr.add(size) {
                     if *cp == 0 {
                         c += 1;
                         cmd.push(get_unchecked_str(cp, start));
@@ -421,9 +488,9 @@ fn update_process(wrap: &Wrap, pid: Pid,
                     }
                     cp = cp.offset(1);
                 }
-                p.cmd = cmd;
+                p.cmd = parse_command_line(&cmd);
                 start = cp;
-                while cp < ptr.offset(size as isize) {
+                while cp < ptr.add(size) {
                     if *cp == 0 {
                         if cp == start {
                             break;
@@ -433,7 +500,7 @@ fn update_process(wrap: &Wrap, pid: Pid,
                     }
                     cp = cp.offset(1);
                 }
-                if need_root == true {
+                if need_root {
                     for env in p.environ.iter() {
                         if env.starts_with("PATH=") {
                             p.root = Path::new(&env[6..]).to_path_buf();
@@ -443,13 +510,44 @@ fn update_process(wrap: &Wrap, pid: Pid,
                 }
             }
         } else {
-            // we don't have enough priviledges to get access to these info
-            return Err(());
+            return Err(()); // not enough rights I assume?
         }
         Ok(Some(p))
     }
 }
 
+fn get_proc_list() -> Option<Vec<Pid>> {
+    let count = unsafe { ffi::proc_listallpids(::std::ptr::null_mut(), 0) };
+    if count < 1 {
+        return None;
+    }
+    let mut pids: Vec<Pid> = Vec::with_capacity(count as usize);
+    unsafe { pids.set_len(count as usize); }
+    let count = count * mem::size_of::<Pid>() as i32;
+    let x = unsafe { ffi::proc_listallpids(pids.as_mut_ptr() as *mut c_void, count) };
+
+     if x < 1 || x as usize >= pids.len() {
+        None
+    } else {
+        unsafe { pids.set_len(x as usize); }
+        Some(pids)
+    }
+}
+
+fn get_arg_max() -> usize {
+    let mut mib: [c_int; 3] = [libc::CTL_KERN, libc::KERN_ARGMAX, 0];
+    let mut arg_max = 0i32;
+    let mut size = mem::size_of::<c_int>();
+    unsafe {
+        if libc::sysctl(mib.as_mut_ptr(), 2, (&mut arg_max) as *mut i32 as *mut c_void,
+                        &mut size, ::std::ptr::null_mut(), 0) == -1 {
+            4096 // We default to this value
+        } else {
+            arg_max as usize
+        }
+    }
+}
+
 impl System {
     fn clear_procs(&mut self) {
         let mut to_delete = Vec::new();
@@ -466,7 +564,7 @@ impl System {
 }
 
 impl SystemExt for System {
-    fn new() -> System {
+    fn new_with_specifics(refreshes: RefreshKind) -> System {
         let mut s = System {
             process_list: HashMap::new(),
             mem_total: 0,
@@ -477,11 +575,12 @@ impl SystemExt for System {
             page_size_kb: unsafe { sysconf(_SC_PAGESIZE) as u64 >> 10 }, // divide by 1024
             temperatures: Vec::new(),
             connection: get_io_service_connection(),
-            disks: get_disks(),
+            disks: Vec::new(),
             network: network::new(),
             uptime: get_uptime(),
+            port: unsafe { ffi::mach_host_self() },
         };
-        s.refresh_all();
+        s.refresh_specifics(refreshes);
         s
     }
 
@@ -499,26 +598,24 @@ impl SystemExt for System {
         unsafe {
             // get system values
             // get swap info
-            let mut xs: ffi::xsw_usage = ::std::mem::zeroed::<ffi::xsw_usage>();
+            let mut xs: ffi::xsw_usage = mem::zeroed::<ffi::xsw_usage>();
             if get_sys_value(ffi::CTL_VM, ffi::VM_SWAPUSAGE,
-                             ::std::mem::size_of::<ffi::xsw_usage>(),
+                             mem::size_of::<ffi::xsw_usage>(),
                              &mut xs as *mut ffi::xsw_usage as *mut c_void, &mut mib) {
                 self.swap_total = xs.xsu_total >> 10; // divide by 1024
                 self.swap_free = xs.xsu_avail >> 10; // divide by 1024
             }
             // get ram info
             if self.mem_total < 1 {
-                get_sys_value(ffi::CTL_HW, ffi::HW_MEMSIZE, ::std::mem::size_of::<u64>(),
+                get_sys_value(ffi::CTL_HW, ffi::HW_MEMSIZE, mem::size_of::<u64>(),
                               &mut self.mem_total as *mut u64 as *mut c_void, &mut mib);
                 self.mem_total >>= 10; // divide by 1024
             }
             let count: u32 = ffi::HOST_VM_INFO64_COUNT;
-            let mut stat = ::std::mem::zeroed::<ffi::vm_statistics64>();
-            if ffi::host_statistics64(ffi::mach_host_self(), ffi::HOST_VM_INFO64,
+            let mut stat = mem::zeroed::<ffi::vm_statistics64>();
+            if ffi::host_statistics64(self.port, ffi::HOST_VM_INFO64,
                                       &mut stat as *mut ffi::vm_statistics64 as *mut c_void,
-                                      &count as *const u32) == ffi::KERN_SUCCESS {
-                //self.mem_free = u64::from(stat.free_count + stat.inactive_count
-                //                          + stat.speculative_count) * self.page_size_kb;
+                                      &count) == ffi::KERN_SUCCESS {
                 // From the apple documentation:
                 //
                 // /*
@@ -527,11 +624,16 @@ impl SystemExt for System {
                 //  * used to hold data that was read speculatively from disk but
                 //  * haven't actually been used by anyone so far.
                 //  */
-                self.mem_free = u64::from(stat.free_count) * self.page_size_kb;
+                // self.mem_free = u64::from(stat.free_count) * self.page_size_kb;
+                self.mem_free = self.mem_total - (u64::from(stat.active_count)
+                                 + u64::from(stat.inactive_count) + u64::from(stat.wire_count)
+                                 + u64::from(stat.speculative_count)
+                                 - u64::from(stat.purgeable_count))
+                                * self.page_size_kb;
             }
 
             if let Some(con) = self.connection {
-                if self.temperatures.len() < 1 {
+                if self.temperatures.is_empty() {
                     // getting CPU critical temperature
                     let mut v = vec!('T' as i8, 'C' as i8, '0' as i8, 'D' as i8, 0);
                     let tmp = get_temperature(con, v.as_mut_ptr());
@@ -596,7 +698,7 @@ impl SystemExt for System {
             if self.processors.is_empty() {
                 let mut num_cpu = 0;
 
-                if !get_sys_value(ffi::CTL_HW, ffi::HW_NCPU, ::std::mem::size_of::<u32>(),
+                if !get_sys_value(ffi::CTL_HW, ffi::HW_NCPU, mem::size_of::<u32>(),
                                   &mut num_cpu as *mut usize as *mut c_void, &mut mib) {
                     num_cpu = 1;
                 }
@@ -604,7 +706,7 @@ impl SystemExt for System {
                 self.processors.push(
                     processor::create_proc("0".to_owned(),
                                            Arc::new(ProcessorData::new(::std::ptr::null_mut(), 0))));
-                if ffi::host_processor_info(ffi::mach_host_self(), ffi::PROCESSOR_CPU_LOAD_INFO,
+                if ffi::host_processor_info(self.port, ffi::PROCESSOR_CPU_LOAD_INFO,
                                        &mut num_cpu_u as *mut u32,
                                        &mut cpu_info as *mut *mut i32,
                                        &mut num_cpu_info as *mut u32) == ffi::KERN_SUCCESS {
@@ -619,7 +721,7 @@ impl SystemExt for System {
                         self.processors.push(p);
                     }
                 }
-            } else if ffi::host_processor_info(ffi::mach_host_self(), ffi::PROCESSOR_CPU_LOAD_INFO,
+            } else if ffi::host_processor_info(self.port, ffi::PROCESSOR_CPU_LOAD_INFO,
                                                &mut num_cpu_u as *mut u32,
                                                &mut cpu_info as *mut *mut i32,
                                                &mut num_cpu_info as *mut u32) == ffi::KERN_SUCCESS {
@@ -665,59 +767,39 @@ impl SystemExt for System {
         if count < 1 {
             return
         }
-        let mut pids: Vec<Pid> = Vec::with_capacity(count as usize);
-        unsafe { pids.set_len(count as usize); }
-        let count = count * ::std::mem::size_of::<Pid>() as i32;
-        let x = unsafe { ffi::proc_listallpids(pids.as_mut_ptr() as *mut c_void, count) };
+        if let Some(pids) = get_proc_list() {
+            let taskallinfo_size = mem::size_of::<libc::proc_taskallinfo>() as i32;
+            let taskinfo_size = mem::size_of::<libc::proc_taskinfo>() as i32;
+            let threadinfo_size = mem::size_of::<libc::proc_threadinfo>() as i32;
+            let arg_max = get_arg_max();
 
-        if x < 1 || x as usize > pids.len() {
-            return
-        } else if pids.len() > x as usize {
-            unsafe { pids.set_len(x as usize); }
+            let entries: Vec<Process> = {
+                let wrap = &Wrap(UnsafeCell::new(&mut self.process_list));
+                pids.par_iter()
+                    .flat_map(|pid| {
+                        let mut mib: [c_int; 3] = [libc::CTL_KERN, libc::KERN_ARGMAX, 0];
+                        match update_process(wrap, *pid, taskallinfo_size, taskinfo_size,
+                                             threadinfo_size, &mut mib, arg_max as size_t) {
+                            Ok(x) => x,
+                            Err(_) => None,
+                        }
+                    })
+                    .collect()
+            };
+            entries.into_iter().for_each(|entry| {
+                self.process_list.insert(entry.pid(), entry);
+            });
+            self.clear_procs();
         }
-
-        let taskallinfo_size = ::std::mem::size_of::<libc::proc_taskallinfo>() as i32;
-        let taskinfo_size = ::std::mem::size_of::<libc::proc_taskinfo>() as i32;
-        let threadinfo_size = ::std::mem::size_of::<libc::proc_threadinfo>() as i32;
-
-        let mut mib: [c_int; 3] = [libc::CTL_KERN, libc::KERN_ARGMAX, 0];
-        let mut arg_max = 0i32;
-        let mut size = ::std::mem::size_of::<c_int>();
-        unsafe {
-            while libc::sysctl(mib.as_mut_ptr(), 2, (&mut arg_max) as *mut i32 as *mut c_void,
-                               &mut size, ::std::ptr::null_mut(), 0) == -1 {}
-        }
-        let entries: Vec<Process> = {
-            let wrap = &Wrap(UnsafeCell::new(&mut self.process_list));
-            pids.par_iter()
-                .flat_map(|pid| {
-                    let mut mib: [c_int; 3] = [libc::CTL_KERN, libc::KERN_ARGMAX, 0];
-                    match update_process(wrap, *pid, taskallinfo_size, taskinfo_size,
-                                         threadinfo_size, &mut mib, arg_max as size_t) {
-                        Ok(x) => x,
-                        Err(_) => None,
-                    }
-                })
-                .collect()
-        };
-        entries.into_iter().for_each(|entry| {
-            self.process_list.insert(entry.pid(), entry);
-        });
-        self.clear_procs();
     }
 
     fn refresh_process(&mut self, pid: Pid) -> bool {
-        let taskallinfo_size = ::std::mem::size_of::<libc::proc_taskallinfo>() as i32;
-        let taskinfo_size = ::std::mem::size_of::<libc::proc_taskinfo>() as i32;
-        let threadinfo_size = ::std::mem::size_of::<libc::proc_threadinfo>() as i32;
+        let taskallinfo_size = mem::size_of::<libc::proc_taskallinfo>() as i32;
+        let taskinfo_size = mem::size_of::<libc::proc_taskinfo>() as i32;
+        let threadinfo_size = mem::size_of::<libc::proc_threadinfo>() as i32;
 
         let mut mib: [c_int; 3] = [libc::CTL_KERN, libc::KERN_ARGMAX, 0];
-        let mut arg_max = 0i32;
-        let mut size = ::std::mem::size_of::<c_int>();
-        unsafe {
-            while libc::sysctl(mib.as_mut_ptr(), 2, (&mut arg_max) as *mut i32 as *mut c_void,
-                               &mut size, ::std::ptr::null_mut(), 0) == -1 {}
-        }
+        let arg_max = get_arg_max();
         match {
             let wrap = Wrap(UnsafeCell::new(&mut self.process_list));
             update_process(&wrap, pid, taskallinfo_size, taskinfo_size,
